"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RestAPI = void 0;
const utils_js_1 = require("../common/utils.js");
const index_js_1 = require("../network/index.js");
const errors_js_1 = require("../sdk/errors.js");
const record_js_1 = require("../sdk/record.js");
const transaction_js_1 = require("../sdk/transaction.js");
const create_api_js_1 = require("./create-api.js");
const utils_js_2 = require("./utils.js");
class RestAPI {
    constructor(token, config) {
        this.relations = {
            find: async ({ pagination, search, transaction }) => {
                const { searchParams } = (0, utils_js_2.createSearchParams)(search);
                const tx = (0, utils_js_2.pickTransaction)(transaction);
                return await this.api.relations.find(searchParams, pagination, tx);
            }
        };
        this.properties = {
            delete: async (id, transaction) => {
                var _a;
                return (_a = this.api) === null || _a === void 0 ? void 0 : _a.properties.delete(id, transaction);
            },
            find: async (searchParams, transaction) => {
                var _a;
                return (_a = this.api) === null || _a === void 0 ? void 0 : _a.properties.find(searchParams, transaction);
            },
            findById: async (id, transaction) => {
                var _a;
                return (_a = this.api) === null || _a === void 0 ? void 0 : _a.properties.findById(id, transaction);
            },
            values: async (id, transaction) => {
                var _a;
                return (_a = this.api) === null || _a === void 0 ? void 0 : _a.properties.values(id, transaction);
            }
        };
        this.labels = {
            find: async (searchParams, transaction) => {
                return this.api.labels.find(searchParams, transaction);
            }
        };
        this.tx = {
            begin: async (config) => {
                var _a;
                const transaction = await ((_a = this.api) === null || _a === void 0 ? void 0 : _a.tx.begin(config));
                const result = new transaction_js_1.Transaction(transaction.data.id);
                result.init(this);
                return result;
            },
            commit: async (id) => { var _a; return (_a = this.api) === null || _a === void 0 ? void 0 : _a.tx.commit(id); },
            get: async (id) => {
                var _a;
                const transaction = await ((_a = this.api) === null || _a === void 0 ? void 0 : _a.tx.get(id));
                const result = new transaction_js_1.Transaction(transaction.data.id);
                result.init(this);
                return result;
            },
            rollback: async (id) => { var _a; return (_a = this.api) === null || _a === void 0 ? void 0 : _a.tx.commit(id); }
        };
        this.fetcher = null;
        if (config === null || config === void 0 ? void 0 : config.httpClient) {
            const url = (0, utils_js_2.buildUrl)(config);
            this.fetcher = (0, index_js_1.createFetcher)({
                httpClient: config.httpClient,
                token,
                url
            });
        }
        this.api = (0, create_api_js_1.createApi)(this.fetcher);
        this.records = {
            attach: async (sourceId, target, options, transaction) => {
                var _a, _b;
                // target is MaybeArray<DBRecordInstance>
                if (target instanceof record_js_1.DBRecordInstance) {
                    const id = (_a = target.data) === null || _a === void 0 ? void 0 : _a.__id;
                    if (id) {
                        return await this.api.records.attach(sourceId, id, options, transaction);
                    }
                    else {
                        throw new errors_js_1.EmptyTargetError('Attach error: Target id is empty');
                    }
                }
                else if ((0, utils_js_1.isArray)(target) && target.every((r) => r instanceof record_js_1.DBRecordInstance)) {
                    const ids = target.map((r) => { var _a; return (_a = r.data) === null || _a === void 0 ? void 0 : _a.__id; }).filter(utils_js_1.toBoolean);
                    if (ids.length) {
                        return await this.api.records.attach(sourceId, ids, options, transaction);
                    }
                    else {
                        throw new errors_js_1.EmptyTargetError('Attach error: Target ids are empty');
                    }
                }
                // target is DBRecordsArrayInstance
                else if (target instanceof record_js_1.DBRecordsArrayInstance) {
                    const ids = (_b = target.data) === null || _b === void 0 ? void 0 : _b.map((r) => r.__id).filter(Boolean);
                    if (ids === null || ids === void 0 ? void 0 : ids.length) {
                        return await this.api.records.attach(sourceId, ids, options, transaction);
                    }
                    else {
                        throw new errors_js_1.EmptyTargetError('Attach error: Target ids are empty');
                    }
                }
                // target is MaybeArray<DBRecord>
                else if ((0, utils_js_1.isObject)(target) && '__id' in target) {
                    return await this.api.records.attach(sourceId, target.__id, options, transaction);
                }
                else if ((0, utils_js_1.isArray)(target) && target.every((r) => (0, utils_js_1.isObject)(r) && '__id' in r)) {
                    const ids = target === null || target === void 0 ? void 0 : target.map((r) => r.__id).filter(Boolean);
                    if (ids === null || ids === void 0 ? void 0 : ids.length) {
                        return await this.api.records.attach(sourceId, ids, options, transaction);
                    }
                    else {
                        throw new errors_js_1.EmptyTargetError('Attach error: Target ids are empty');
                    }
                }
                // target is MaybeArray<string>
                else {
                    return await this.api.records.attach(sourceId, target, options, transaction);
                }
            },
            create: async (labelOrData, maybeDataOrTransaction, transaction) => {
                var _a, _b;
                let response;
                if (labelOrData instanceof record_js_1.DBRecordDraft) {
                    response = await ((_a = this.api) === null || _a === void 0 ? void 0 : _a.records.create(labelOrData, (0, utils_js_2.pickTransaction)(maybeDataOrTransaction)));
                }
                if (!response && (0, utils_js_1.isString)(labelOrData)) {
                    if ((0, utils_js_1.isObjectFlat)(maybeDataOrTransaction)) {
                        const normalizedRecord = (0, utils_js_2.normalizeRecord)({
                            label: labelOrData,
                            payload: maybeDataOrTransaction
                        });
                        response = await ((_b = this.api) === null || _b === void 0 ? void 0 : _b.records.create(new record_js_1.DBRecordDraft(normalizedRecord), transaction));
                    }
                    else if ((0, utils_js_1.isObject)(maybeDataOrTransaction)) {
                        throw Error('Provided data is not a flat object. Consider to use `createMany` method.');
                    }
                }
                if ((response === null || response === void 0 ? void 0 : response.success) && (response === null || response === void 0 ? void 0 : response.data)) {
                    const result = new record_js_1.DBRecordInstance(response.data);
                    result.init(this);
                    return result;
                }
                return new record_js_1.DBRecordInstance();
            },
            createMany: async (labelOrData, maybeDataOrTransaction, transaction) => {
                var _a, _b;
                let response;
                if (labelOrData instanceof record_js_1.DBRecordsBatchDraft) {
                    response = await ((_a = this.api) === null || _a === void 0 ? void 0 : _a.records.createMany(labelOrData, (0, utils_js_2.pickTransaction)(maybeDataOrTransaction)));
                }
                if (!response &&
                    (0, utils_js_1.isString)(labelOrData) &&
                    ((0, utils_js_1.isArray)(maybeDataOrTransaction) || (0, utils_js_1.isObject)(maybeDataOrTransaction))) {
                    const data = new record_js_1.DBRecordsBatchDraft({
                        label: labelOrData,
                        payload: maybeDataOrTransaction
                    });
                    response = await ((_b = this.api) === null || _b === void 0 ? void 0 : _b.records.createMany(data, transaction));
                }
                if ((response === null || response === void 0 ? void 0 : response.success) && (response === null || response === void 0 ? void 0 : response.data)) {
                    const result = new record_js_1.DBRecordsArrayInstance(response.data, response.total);
                    result.init(this);
                    return result;
                }
                return new record_js_1.DBRecordsArrayInstance([]);
            },
            delete: async (searchParams, transaction) => {
                var _a;
                if ((0, utils_js_1.isEmptyObject)(searchParams.where)) {
                    throw new errors_js_1.EmptyTargetError(`You must specify criteria to delete records. Empty criteria are not allowed. If this was intentional, use the Dashboard instead.`);
                }
                return (_a = this.api) === null || _a === void 0 ? void 0 : _a.records.delete(searchParams, transaction);
            },
            deleteById: async (idOrIds, transaction) => {
                var _a;
                return (_a = this.api) === null || _a === void 0 ? void 0 : _a.records.deleteById(idOrIds, transaction);
            },
            detach: async (sourceId, target, options, transaction) => {
                var _a, _b;
                // target is MaybeArray<DBRecordInstance>
                if (target instanceof record_js_1.DBRecordInstance) {
                    const id = (_a = target.data) === null || _a === void 0 ? void 0 : _a.__id;
                    if (id) {
                        return await this.api.records.detach(sourceId, id, options, transaction);
                    }
                    else {
                        throw new errors_js_1.EmptyTargetError('Detach error: Target id is empty');
                    }
                }
                else if ((0, utils_js_1.isArray)(target) && target.every((r) => r instanceof record_js_1.DBRecordInstance)) {
                    const ids = target.map((r) => { var _a; return (_a = r.data) === null || _a === void 0 ? void 0 : _a.__id; }).filter(Boolean);
                    if (ids.length) {
                        return await this.api.records.detach(sourceId, ids, options, transaction);
                    }
                    else {
                        throw new errors_js_1.EmptyTargetError('Detach error: Target ids are empty');
                    }
                }
                // target is DBRecordsArrayInstance
                else if (target instanceof record_js_1.DBRecordsArrayInstance) {
                    const ids = (_b = target.data) === null || _b === void 0 ? void 0 : _b.map((r) => r.__id).filter(Boolean);
                    if (ids === null || ids === void 0 ? void 0 : ids.length) {
                        return await this.api.records.detach(sourceId, ids, options, transaction);
                    }
                    else {
                        throw new errors_js_1.EmptyTargetError('Detach error: Target ids are empty');
                    }
                }
                // target is MaybeArray<DBRecord>
                else if ((0, utils_js_1.isObject)(target) && '__id' in target) {
                    return await this.api.records.detach(sourceId, target.__id, options, transaction);
                }
                else if ((0, utils_js_1.isArray)(target) && target.every((r) => (0, utils_js_1.isObject)(r) && '__id' in r)) {
                    const ids = target === null || target === void 0 ? void 0 : target.map((r) => r.__id).filter(Boolean);
                    if (ids === null || ids === void 0 ? void 0 : ids.length) {
                        return await this.api.records.detach(sourceId, ids, options, transaction);
                    }
                    else {
                        throw new errors_js_1.EmptyTargetError('Detach error: Target ids are empty');
                    }
                }
                // target is MaybeArray<string>
                else {
                    return await this.api.records.detach(sourceId, target, options, transaction);
                }
            },
            export: async (searchParams, transaction) => {
                var _a;
                return (_a = this.api) === null || _a === void 0 ? void 0 : _a.records.export(searchParams, transaction);
            },
            find: async (labelOrSearchParams, searchParamsOrTransaction, transaction) => {
                var _a;
                const isTransactionParam = (0, utils_js_2.isTransaction)(searchParamsOrTransaction);
                const { id, searchParams } = (0, utils_js_2.createSearchParams)(labelOrSearchParams, searchParamsOrTransaction);
                const tx = isTransactionParam ? searchParamsOrTransaction : transaction;
                const response = await ((_a = this.api) === null || _a === void 0 ? void 0 : _a.records.find({ id, searchParams }, tx));
                const result = new record_js_1.DBRecordsArrayInstance(response.data, response.total, searchParamsOrTransaction);
                result.init(this);
                return result;
            },
            findById: async (idOrIds, transaction) => {
                var _a, _b;
                if ((0, utils_js_1.isArray)(idOrIds)) {
                    const response = (await ((_a = this.api) === null || _a === void 0 ? void 0 : _a.records.findById(idOrIds, transaction)));
                    const result = new record_js_1.DBRecordsArrayInstance(response.data, response.total);
                    result.init(this);
                    return result;
                }
                else {
                    const response = (await ((_b = this.api) === null || _b === void 0 ? void 0 : _b.records.findById(idOrIds, transaction)));
                    const result = new record_js_1.DBRecordInstance(response.data);
                    result.init(this);
                    return result;
                }
            },
            findOne: async (labelOrSearchParams, searchParamsOrTransaction, transaction) => {
                var _a;
                const isTransactionParam = (0, utils_js_2.isTransaction)(searchParamsOrTransaction);
                const { searchParams } = (0, utils_js_2.createSearchParams)(labelOrSearchParams, searchParamsOrTransaction);
                const tx = isTransactionParam ? searchParamsOrTransaction : transaction;
                const response = await ((_a = this.api) === null || _a === void 0 ? void 0 : _a.records.findOne(searchParams, tx));
                const result = new record_js_1.DBRecordInstance(response.data, searchParamsOrTransaction);
                result.init(this);
                return result;
            },
            findUniq: async (labelOrSearchParams, searchParamsOrTransaction, transaction) => {
                var _a;
                const isTransactionParam = (0, utils_js_2.isTransaction)(searchParamsOrTransaction);
                const { searchParams } = (0, utils_js_2.createSearchParams)(labelOrSearchParams, searchParamsOrTransaction);
                const tx = isTransactionParam ? searchParamsOrTransaction : transaction;
                const response = await ((_a = this.api) === null || _a === void 0 ? void 0 : _a.records.findUniq(searchParams, tx));
                const result = new record_js_1.DBRecordInstance(response.data, searchParamsOrTransaction);
                result.init(this);
                return result;
            },
            properties: async (id, transaction) => {
                var _a;
                return (_a = this.api) === null || _a === void 0 ? void 0 : _a.records.properties(id, transaction);
            },
            relations: async (id, transaction) => {
                return await this.api.records.relations(id, transaction);
            },
            set: async (id, data, transaction) => {
                var _a, _b;
                let response;
                if (data instanceof record_js_1.DBRecordDraft) {
                    response = await ((_a = this.api) === null || _a === void 0 ? void 0 : _a.records.set(id, data, transaction));
                }
                else if ((0, utils_js_1.isObjectFlat)(data)) {
                    const properties = (0, utils_js_2.prepareProperties)(data);
                    response = await ((_b = this.api) === null || _b === void 0 ? void 0 : _b.records.set(id, new record_js_1.DBRecordDraft({ properties }), transaction));
                }
                else if ((0, utils_js_1.isObject)(data)) {
                    throw Error('Provided data is not a flat object. Consider to use `createMany` method.');
                }
                if ((response === null || response === void 0 ? void 0 : response.success) && (response === null || response === void 0 ? void 0 : response.data)) {
                    const result = new record_js_1.DBRecordInstance(response.data);
                    result.init(this);
                    return result;
                }
                return new record_js_1.DBRecordInstance();
            },
            update: async (id, data, transaction) => {
                var _a, _b;
                let response;
                if (data instanceof record_js_1.DBRecordDraft) {
                    response = await ((_a = this.api) === null || _a === void 0 ? void 0 : _a.records.update(id, data, transaction));
                }
                else if ((0, utils_js_1.isObjectFlat)(data)) {
                    const properties = (0, utils_js_2.prepareProperties)(data);
                    const recordDraft = new record_js_1.DBRecordDraft({ properties });
                    response = await ((_b = this.api) === null || _b === void 0 ? void 0 : _b.records.update(id, recordDraft, transaction));
                }
                else if ((0, utils_js_1.isObject)(data)) {
                    throw Error('Provided data is not a flat object. Consider to use `createMany` method.');
                }
                if ((response === null || response === void 0 ? void 0 : response.success) && (response === null || response === void 0 ? void 0 : response.data)) {
                    const result = new record_js_1.DBRecordInstance(response.data);
                    result.init(this);
                    return result;
                }
                return new record_js_1.DBRecordInstance();
            }
        };
    }
}
exports.RestAPI = RestAPI;
